<?php

namespace Drupal\deprecation_status\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\deprecation_status\DataSource;

class GraphController extends ControllerBase {

  public function graphPage() {
    // git checkout `git rev-list -1 --before="Nov 14 2019" 9.0.x`

    // $ grep -ro "@deprecated" core | wc -l 
    // $ grep -r "@deprecated" core | grep -o "10.0.0" | wc -l
    
    // $ grep -ro "E_USER_DEPRECATED" core | wc -l
    // $ grep -r "E_USER_DEPRECATED" core | grep -o "10.0.0" | wc -l

    // Read historic core deprecation count data.
    $at_deprecated = '';
    $e_user_deprecated = '';
    $file = fopen(DataSource::getFullPath('core_deprecation_count.csv'), 'r');
    $i = 0;
    while ($line = fgetcsv($file, 0, ";")) {
      // Skip the header row.
      if ($i > 0) {
        $at_deprecated .= '{x: new Date("' . $line[0] . '"), y: ' . $line[1] . '},';
        $e_user_deprecated .= '{x: new Date("' . $line[0] . '"), y: ' . $line[2] . '},';
      }
      $i++;
    }
    fclose($file);
    $at_deprecated = trim($at_deprecated, ',');
    $e_user_deprecated = trim($e_user_deprecated, ',');

    $build['core'] = [
      '#markup' => '<h2>Deprecated APIs in Drupal 9.0.x core</h2><p>A key goal of Drupal 9 is to remove deprecated APIs. Tracking the removal of them based on @deprecated and E_USER_DEPRECATED mentions, until the planned beta date on the week of March 2, 2020.</p>',
      '#children' => <<<COREEND
  <script src="https://cdn.jsdelivr.net/npm/moment@2.24.0/moment.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/chart.js@2.8.0"></script>
  <canvas id="coreDeprecated"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('coreDeprecated').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [{
        label: 'Drupal core @deprecated mentions',
        borderColor: '#2586a4',
        borderWidth: 4,
        data: [$at_deprecated],
        cubicInterpolationMode: 'monotone',
        fill: false,
        pointRadius: 4,
        pointHoverRadius: 10,
        pointBackgroundColor: '#2586a4',
      },
      {
        label: 'Drupal core E_USER_DEPRECATED mentions',
        borderColor: '#f3a935',
        borderWidth: 4,
        data: [$e_user_deprecated],
        cubicInterpolationMode: 'monotone',
        fill: false,
        pointRadius: 4,
        pointHoverRadius: 10,
        pointBackgroundColor: '#f3a935',
      }]
    },
    options: {
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-03-02')
          }
        }],
        yAxes: [{
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
COREEND
    ];

    // =========================================================================

    // Read historic project data.
    $projects = [];
    $legend = [];
    $file = fopen(DataSource::getFullPath('project_breakdown.csv'), 'r');
    $i = 0;
    while ($line = fgetcsv($file, 0, ";")) {
      if ($i == 0) {
        $legend = $line;
        array_shift($legend);
      }
      else {
        $time = array_shift($line);
        foreach($legend as $index => $label) {
          $projects[$label][$time] = $line[$index];
        }
      }
      $i++;
    }
    fclose($file);

    $breakdown_colors = [
      'No problems found' => "#6ebe9f",
      'All problems fixable now' => "#2586a4",
      'Some problems fixable now' => "#f3a935",
      'Needs manual review' => "#55596a",
      'Only problems fixable later' => "#c73558",
    ];

    $datasets = [];
    foreach($projects as $label => $dataset) {
      $raw = '{label: "'. $label . '", data: [';
      foreach($dataset as $time => $value) {
        $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
      }
      $raw = trim($raw, ',') . ']';
      if (isset($breakdown_colors[$label])) {
        $raw .= ', borderColor: "' . $breakdown_colors[$label] . '", backgroundColor: "' . $breakdown_colors[$label] . '"';
      }
      $raw .= '}';
      $datasets[] = $raw;
    }
    $datasets = join(',', $datasets);

    $build['projects'] = [
      '#markup' => '<h2>All drupal.org projects by fixability</h2><p>Ideally projects with no problems found would increase, while every other type of project would decrease as time goes on.</p>',
      '#children' => <<<PROJECTSEND
  <canvas id="projectStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('projectStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-06-03')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
PROJECTSEND
    ];

    // =========================================================================

    $legend_colors = [
      'Fix now' => "#f3a935",
      'Once 8.6 unsupported (archive)' => "#55596a",
      'Once 8.7 unsupported' => "#c73558",
      'Uncategorized' => "",
    ];
    $legend = array_keys($legend_colors);

    // Read historic error data.
    $errors = [];
    $file = fopen(DataSource::getFullPath('error_breakdown.csv'), 'r');
    $i = 0;
    while ($line = fgetcsv($file, 0, ";")) {
      if ($i > 0) {
        $time = array_shift($line);
        foreach($legend as $index => $label) {
          $errors[$label][$time] = $line[$index];
        }
      }
      $i++;
    }
    fclose($file);

    $datasets = [];
    foreach($errors as $label => $dataset) {
      $raw = '{label: "'. $label . '", data: [';
      foreach($dataset as $time => $value) {
        $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
      }
      $raw = trim($raw, ',') . ']';
      if (!empty($legend_colors[$label])) {
        $raw .= ', borderColor: "' . $legend_colors[$label] . '", backgroundColor: "' . $legend_colors[$label] . '"';
      }
      $raw .= '}';
      $datasets[] = $raw;
    }
    $datasets = join(',', $datasets);

    $build['errors'] = [
      '#markup' => '<h2>All drupal.org project errors by fixability</h2><p>Errors in all categories may be decreasing but errors not yet cleanly fixable will likely not decrease for a while.</p>',
      '#children' => <<<ERRORSEND
  <canvas id="errorStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('errorStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-06-03')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
ERRORSEND
    ];

    // =========================================================================

    // Read top error data.
    $top_errors = [];
    $times = [];
    $file = fopen(DataSource::getFullPath('top_error_breakdown.csv'), 'r');
    $i = 0;
    while ($line = fgetcsv($file, 0, ";")) {
      if ($i == 0) {
        $times = $line;
        array_shift($times);
      }
      if ($i > 0) {
        foreach ($times as $index => $time) {
          $top_errors[$line[0]][$time] = $line[$index+1];
        }
      }
      $i++;
    }
    fclose($file);

    $colors = ['#e6194b', '#3cb44b', '#ffe119', '#4363d8', '#f58231', '#911eb4', '#46f0f0', '#f032e6', '#bcf60c', '#fabebe', '#008080', '#e6beff', '#9a6324', '#fffac8', '#800000', '#aaffc3', '#808000'];
    $datasets = [];
    foreach($top_errors as $label => $dataset) {
      list($short_label) = explode('. ', $label);
      $short_label = preg_replace('!Call to deprecated method (.+) of class (.+)!', 'Call to \2::\1', $short_label);
      $short_label = preg_replace('!Call to deprecated function (.+)!', 'Call to \1', $short_label);
      if ($short_label == 'Class [redacted] extends deprecated class Drupal\simpletest\WebTestBase') {
        $short_label = 'Extending Drupal\simpletest\WebTestBase';
      }
      elseif ($short_label == 'Call to method setUp() of deprecated class Drupal\simpletest\WebTestBase') {
        $short_label = 'Call to Drupal\simpletest\WebTestBase::setUp()';
      }
      $short_label = addslashes($short_label);

      $raw = '{label: "'. $short_label . '", fill: true, data: [';
      foreach($dataset as $time => $value) {
        $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
      }
      $raw = trim($raw, ',') . ']';
      $color = array_shift($colors);
      $raw .= ', borderColor: "' . $color . '", backgroundColor: "' . $color . '"';
      $raw .= '}';
      $datasets[] = $raw;
    }
    $datasets = join(',', $datasets);

    $build['toperrors'] = [
      '#markup' => '<h2>Top 15 errors across all drupal.org projects</h2><p>These top errors originally accounted for almost two thirds of all deprecated API use. Fixing them across projects would solve most Drupal 9 compatibility gaps.</p>',
      '#children' => <<<ERRORSEND
  <canvas id="topErrorStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('topErrorStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-06-03')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
ERRORSEND
    ];

    return $build;
  }

}
